
require_relative "config.rb"
require "rake"
require 'fileutils'
require "set"
require "pathname"
class Pathname
  def fileName
    if not extname.nil?
      return basename.to_s[0..(basename.to_s.length-extname.length-1)]
    end
    return basename.to_s
  end
end

module QCloudHive
  class BuildShell
    attr_accessor :productsDir
    attr_accessor :config
    attr_accessor :name
    attr_accessor :other_cflags
    attr_accessor :sdk
    def to_cmd
      cmd ="xcodebuild -configuration #{config} -sdk #{sdk} -workspace #{name}.xcworkspace -scheme #{name} clean\n"
      cmd +="xcodebuild DEPLOYMENT_POSTPROCESSING=YES STRIP_STYLE=debugging "
      if not other_cflags.nil?
        cmd += " OTHER_CFLAGS=#{other_cflags}"
      end
      cmd +=  " -configuration  #{config}"
      cmd += " -workspace #{name}.xcworkspace -scheme #{name}"
      cmd +=   "  -sdk #{sdk}  -derivedDataPath #{productsDir}  "
      cmd
    end
  end
  def QCloudHive.beginShell(path, shell)
    Rake::sh "cd #{path}\n #{shell}"
  end

  def QCloudHive.UnlinkedModule(name, hiveXcodeProject)
    podspec = hiveXcodeProject.hiveSpecModules.select { |s|
      s.name == name
    }.first
    if podspec.nil?
      Error "您不想链接的模块#{name}, 不存在本地"
    end
    unlinked = []
    L.info "$$$$$$$$#{podspec.path}"
    if not podspec.path.nil?
      pod = Pod::Specification.from_file(podspec.path)
      vendored_libraries = pod.attributes_hash["vendored_libraries"]
      vendoredFrameworks = pod.attributes_hash["vendored_frameworks"]
      if not vendoredFrameworks.nil?
        vendoredFrameworks.each { |e|
          unlinked << Pathname.new(e).fileName
        }
      end
      if not vendored_libraries.nil?
        vendored_libraries.each { |e|
          name = Pathname.new(e).fileName
          if name.start_with?("lib")
            name = name[3..-1]
          end
          unlinked << name
        }
      end
      L.info "Unlinked #{unlinked}"
      return unlinked
    end
  end
  def QCloudHive.BuildFramework(cmd, opts)
    name = opts[:i_name]
    if name == nil
      L.error "ERROR! no project name, please input one"
      L.error cmd.help
      exit(1)
    end
    buildSimArch = opts[:i_build_sim]
    if buildSimArch.nil?
      buildSimArch = true
    end
    L.info "build sim #{buildSimArch}"
    unlinked = opts[:i_unlinked_frameworks]
    if unlinked.nil?
      unlinked = []
    end
    L.info "un linked frameworks #{unlinked}"
    output = Dir.pwd
    rootShellPath = Config.scriptesDirectory
    Rake::sh "#{rootShellPath}HiveFramework #{name}"

    projectDir = "#{output}/#{name}"
    project = Xcodeproj::Project.open("#{output}/#{name}/#{name}.xcodeproj")
    project.targets.each do |target|
      if target.name == name
        target.build_configurations.each do |config|
          config.build_settings['ENABLE_BITCODE'] = 'NO'
          puts "set bitcode NO"
        end
      end
      puts "target name #{name}"
    end

    project.save

    Rake::sh "cd #{name} \n" +
    "pod update\n"
    # delete unlinked frameworks
    project = Xcodeproj::Project.open("#{output}/#{name}/#{name}.xcodeproj")
    xcconfigFiles = project.files.select{ |f|
      L.info "file #{f.path}"
      f.path.end_with?(".xcconfig")
    }
    L.info("config files #{xcconfigFiles}")
    hiveXcodeProject = QCloudHive.xcodeprojectByPath(Pathname.new("#{output}/#{name}").realpath.to_s)
    xcconfigFiles.each { |f|
      path = Pathname.new(projectDir).join(f.path)
      config = Xcodeproj::Config.new(path)
      linked = config.other_linker_flags[:libraries].to_a
      L.info "#{config.other_linker_flags}"
      frameworkLinked = config.other_linker_flags[:frameworks].to_a
      L.info "linked #{linked}"
      # unlinked.each { |e|
      #   linked.delete e
      #   willUnlink = UnlinkedModule(e, hiveXcodeProject)
      #   L.info "will unlink #{willUnlink}"
      #   linked -= willUnlink
      #   frameworkLinked -= willUnlink
      # }
      linked = linked.compact
      frameworkLinked = frameworkLinked.compact
      config.other_linker_flags[:libraries] = linked.to_set
      config.other_linker_flags[:frameworks] = frameworkLinked.to_set
      L.info "final linked #{linked}"
      config.save_as(path)
    }
    #

    productsDir = "Products"
    productsDir = "#{projectDir}/#{productsDir}"
    if not Pathname.new(productsDir).exist?
      Pathname.new(productsDir).mkdir
    end



    buildCMD = BuildShell.new()
    buildCMD.config = "Release"
    buildCMD.productsDir = productsDir
    buildCMD.other_cflags = "\"-fembed-bitcode\""
    buildCMD.name = name
    buildCMD.sdk = "iphoneos"
    beginShell(name,buildCMD.to_cmd)


    deviceFramework="#{productsDir}/Build/Products/#{buildCMD.config}-iphoneos/#{name}.framework"

    if buildSimArch == true
      buildCMD = BuildShell.new()
      buildCMD.config = "Release"
      buildCMD.productsDir = productsDir
      buildCMD.other_cflags = "\"-fembed-bitcode\""
      buildCMD.name = name
      buildCMD.sdk = "iphonesimulator"

      beginShell(name,buildCMD.to_cmd)
      simFramework="#{productsDir}/Build/Products/#{buildCMD.config}-iphonesimulator/#{name}.framework"
      Rake::sh " lipo -create #{deviceFramework}/#{name} #{simFramework}/#{name} -output #{deviceFramework}/#{name}"

    end

    puts "Copy frameworks"
    outputFramework = "#{name}.framework"
    outputFrameworkPath = Pathname.new(outputFramework)
    if outputFrameworkPath.exist?
      FileUtils.rm_rf(outputFrameworkPath.to_path)
    end
    Rake::sh "cp -rvf #{deviceFramework} #{outputFramework}"
    Rake::sh "cp #{name}/.hivemodules #{outputFramework}/"
    headersPath = Pathname.new("#{outputFramework}/Headers/")
    if not headersPath.exist?
      headersPath.mkdir
    end
    if Pathname("#{name}/Pods/Headers/Public").exist?
      shell = "find #{name}/Pods/Headers/Public -name \"*.h\" -exec cp {}  #{outputFramework}/Headers/  \\;"
      Rake::sh shell
    end
    willRmBuild = opts[:i_rm_build]
    if willRmBuild != false
      FileUtils.rm_rf("#{name}")
    end
  end
end
